from django.core.management.base import BaseCommand, CommandError

from document_management.openkm.sync import SyncFolderList

class DummyDocument:
    path = ''

class Command(BaseCommand):
    help = """
    Updates the Django database with the document meta data from OpenKM

    args:
    construct_categories - dynamically creates all the categories on OpenKM from Django models
    """

    args = ('construct_categories', 'sync_local_folder_list')

    def handle(self, *args, **options):
        # handle arguments first
        if 'sync_local_folder_list' in args:
            self.sync_local_folder_list()
        else:
            logger.info("No arguments given")

    def sync_local_folder_list(self):
        """
        Updates the local database table of OpenKM folder metadata
        """
        folder_list = SyncFolderList()
        folder_list.execute()











import base64
import logging

import suds

"""
Some useful helper and decorator functions
"""

def try_except(fn):
    """ A decorator to catch suds exceptions """
    def wrapped(*args, **kwargs):
        try:
            fn(*args, **kwargs)
        except suds.WebFault as detail:
            return detail
    return wrapped

def make_file_java_byte_array_compatible(file_obj):
    """ 
    Reads in a file and converts it to a format accepted as Java byte array 
    :param file object
    :return string
    """
    encoded_data = base64.b64encode(file_obj.read())
    strg = ''
    for i in xrange((len(encoded_data)/40)+1):
        strg += encoded_data[i*40:(i+1)*40]
        
    return strg

def java_byte_array_to_binary(file_obj):
    """ 
    Converts a java byte array to a binary stream
    :param java byte array as string (pass in as a file like object, can use StringIO)
    :return binary string
    """
    decoded_data = base64.b64decode(file_obj.read())
    strg = ''
    for i in xrange((len(decoded_data)/40)+1):
        strg += decoded_data[i*40:(i+1)*40]
        
    return strg

def find_key(dic, val):
    """return the key of dictionary dic given the value"""
    return [k for k, v in dic.iteritems() if v == val][0]

logger = logging.getLogger('tce')
logger.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(levelname)-8s %(message)s')




from .models import OpenKmFolderList, OpenKmDocument
from django.contrib import admin

class OpenKmDocumentAdmin(admin.ModelAdmin):
    readonly_fields = ['author', 'created', 'path', 'permissions', 'subscribed', 'uuid', 'filename']

admin.site.register(OpenKmFolderList)
admin.site.register(OpenKmDocument, OpenKmDocumentAdmin)


#!/usr/bin/env python
"""
Installation script:

To release a new version to PyPi:
- Ensure the version is correctly set in django-openkm.__init__.py
- Run: python setup.py sdist upload
"""

from setuptools import setup, find_packages

from django_openkm import get_version


setup(name='django_openkm',
      version=get_version().replace(' ', '-'),
      url='https://github.com/igniteflow/django-openkm',
      author="Phil Tysoe",
      author_email="philip.tysoe@tangentlabs.co.uk",
      description="A client library for OpenKM document management system.  Integration with Django 1.2+",
      long_description="A client library for OpenKM document management system.  Integration with Django 1.2+",
      keywords="OpenKM, Django, document management",
      license='BSD',
      platforms=['linux'],
      packages=find_packages(exclude=["*.tests"]),
      install_requires=[
          'suds>=0.4',
          ],
      # See http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=['Environment :: Web Environment',
                   'Framework :: Django',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: BSD License',
                   'Operating System :: Unix',
                   'Programming Language :: Python']
      )




from django.conf import settings

from .client import Auth, Document, Folder, Repository, Property
from .utils import make_file_java_byte_array_compatible

class Session(object):

    def __init__(self):
        self.auth = Auth()

    def open(self):
        self.auth.login()
        return self.auth.token

    def close(self):
        self.auth.logout()

class Category(object):

    def __init__(self):
        self.folder = Folder()
        self.repository = Repository()
        self.property = Property()

    def add_to_node(self, node_path, category_uuid):
        self.property.add_category(node_path, category_uuid)

    def remove_from_node(self, node_path, category_uuid):
        self.property.remove_category(node_path, category_uuid)

    def create(self, new_category_path):
        """ Creates a new category """
        new_category = self.folder.new()
        new_category.path = new_category_path
        return self.folder.create(new_category)

    def remove(self, path):
        """ Removes an existing category """
        return self.folder.delete(path)

    def get_category_root(self):
        """
        :return folder object
        """
        return self.repository.get_categories_folder()

    def get_child_categories(self, path):
        """ Returns the child categories for a given parent folder """
        return self.folder.get_children(path)

    def construct_valid_path_string(self, base_path, new_category_name):
        """
        Utility function to construct a category path from a path
        as returned by OpenKM and a string specifying a new category name
        :base_path string
        :new_category_name string
        :return string
        """
        return "%s/%s" % (base_path, new_category_name)


class Keyword(object):

    def __init__(self):
        self.property = Property()
        self.document = Document()

    def add(self, path, keyword):
        return self.property.add_keyword(path, keyword)

    def remove(self, path, keyword):
        return self.property.remove_keyword(path, keyword)

    def get_for_document(self, path):
        """
        Returns a list of keywords
        :param path: OpenKM node path of document
        """
        document = self.document.get_properties(path)
        return document.keywords


class FileSystem(object):
    """
    File system related functionality
    """
    def get_node_from_path(self, path):
        """ Returns the file name or folder name from a given path """
        return path.split("/")[-1]

    def names_as_list(self, collection):
        """ Generic utility function """
        names = []
        for child in collection.item:
            category_name = self.get_node_from_path(child.path)
            names.append(category_name)

        return names

    def folder_names_as_list(self, collection_of_child_folders):
        return self.names_as_list(collection_of_child_folders)

    def file_names_as_list(self, collection_of_child_documents):
        return self.names_as_list(collection_of_child_documents)

    def get_filename_from_path(self, path):
        """
        Split a file path on the separator and return the file name
        :return string file name
        """
        return path.split("/")[-1]


class DirectoryListing(object):

    documents = []
    folders = []

    def __init__(self):
        self.doc = Document()
        self.folder = Folder()

    def get_root_path(self):
        return settings.OPENKM['UploadRoot']

    def traverse(self, path=None):
        """
        Traverse files and folders
        Returns a list of document objects
        """

        if path is None:
            path = self.get_root_path()

        # get files and add them to the file list
        results = self.doc.get_children(path)

        try:
            for result in results[0]:
                self.documents.append(result)
        except:
            pass

        # get child folders of the path given as param
        folders = self.folder.get_children(path)

        # if there are child folders, traverse them recursively
        try:
            if isinstance(folders[0], list) and len(folders[0]) > 0:
                for folder in folders[0]:
                    self.traverse(path=folder.path)
        except:
            pass

        return self.documents

    def traverse_folders(self, path):
        folders_temp = self.folder.get_children(path)

        # if there are child folders, traverse them recursively
        try:
            if isinstance(folders_temp[0], list) and len(folders_temp[0]) > 0:
                for folder in folders_temp[0]:
                    self.folders.append(folder)
                    self.traverse_folders(path=folder.path)
        except:
            pass
        return self.folders

class DocumentManager(object):

    def __init__(self):
        self.document = Document()

    def create(self, file_obj):
        document = self.document.new()
        document.path = self.create_path_from_filename(file_obj)
        content = self.convert_file_content_to_binary_for_transport(file_obj)
        return self.create_document_on_openkm(document, content)

    def create_path_from_filename(self, file_obj):
        """
        Constructs a path name of the format:

            upload_root + filename

        :returns string:  path name
        """
        return settings.OPENKM['UploadRoot'] + file_obj.__str__()

    def convert_file_content_to_binary_for_transport(self, file_obj):
        return make_file_java_byte_array_compatible(file_obj)

    def create_document_on_openkm(self, document, content):
        return self.document.create(document, content)

